/**
 * File:
 *  KnightRider.c
 *
 * Notes:
 *  Simple demo created for the easyPic7 board.
 *
 * History:
 *  29/04/2012 Written by Simon Platten
 */
#include "fader.h"

// Constants
#define TIMER_FREQUENCY         ((uint)5000) // Hz
#define DELAY_BETWEEN_LEDS      (TIMER_FREQUENCY / 12)

typedef struct {
  byte     bPin;                       // The pin to send to the port
  uint     uintPreDelay;               // The count to wait before applying
  uint     uintPostDelay;              // The count to wait after applying
  faderDef fader;                      // Fader
} outputScheduleDef;

typedef struct {
  byte*              pPort;            // The port the output is associated with
  byte               bTotalSteps;      // The number of steps in the animation
  unsigned           uLoop : 1;        // Flag to loop
  unsigned           uStop : 1;        // Flag to indicate animation complete
  byte               bIndex;           // Place in sequence
  byte               bStage;           // Stage in current animation
  uint               uintCtr;          // Current delay value
} animDef;

static volatile outputScheduleDef aryLEDseq[] = {
  { 0, 0, DELAY_BETWEEN_LEDS },
  { 1, 0, DELAY_BETWEEN_LEDS },
  { 2, 0, DELAY_BETWEEN_LEDS },
  { 3, 0, DELAY_BETWEEN_LEDS },
  { 4, 0, DELAY_BETWEEN_LEDS },
  { 5, 0, DELAY_BETWEEN_LEDS },
  { 6, 0, DELAY_BETWEEN_LEDS },
  { 7, 0, DELAY_BETWEEN_LEDS },
// Bounce back
  { 6, 0, DELAY_BETWEEN_LEDS },
  { 5, 0, DELAY_BETWEEN_LEDS },
  { 4, 0, DELAY_BETWEEN_LEDS },
  { 3, 0, DELAY_BETWEEN_LEDS },
  { 2, 0, DELAY_BETWEEN_LEDS },
  { 1, 0, DELAY_BETWEEN_LEDS } };

static volatile animDef animTest = { &LATC, 
                     (sizeof(aryLEDseq) / sizeof(aryLEDseq[0])), 1 };
static outputScheduleDef* pCurAnim;

/**
 * Function:
 *  playAnimation
 *
 * Parameters:
 *  pAnim, pointer to animation and sequence
 *
 * Returns:
 *  none
 */
void playAnimation(animDef* pAnim) {
  if ( pAnim->uStop ) {
// Animation complete and stopped, do nothing
     return;
  }
// Get the current step in the animation
  if ( pAnim->bIndex >=  pAnim->bTotalSteps ) {
    if ( !pAnim->uLoop ) {
// No loop, flag that animation is complete and abort
      pAnim->uStop = 1;
      return;
    }
// Reset index and start over
    pAnim->bIndex = 0;
  }
  pCurAnim = &aryLEDseq[pAnim->bIndex];

  switch((int)pAnim->bStage) {
  case 0:
// Need to initialise this step in the animation
    if ( pCurAnim->uintPreDelay == 0 ) {
// No pre-action delay, go straight to applying mask
      pAnim->bStage = 2;
      break;
    }
    pAnim->uintCtr = pCurAnim->uintPreDelay;
    pAnim->bStage = 1;
    break;
  case 1:
// Pre-action delay
    if ( --pAnim->uintCtr == 0 ) {
      pAnim->bStage = 2;
    } else {
      break;
    }
  case 2:
// Apply mask
    *pAnim->pPort = (1 << pCurAnim->bPin);
// Start the fader
    pCurAnim->fader->intStep = pCurAnim->fader->uintSteps - 1;
    pCurAnim->fader->uStop = 0;

    if ( pCurAnim->uintPostDelay > 0 ) {
      pAnim->uintCtr = pCurAnim->uintPostDelay;
      pAnim->bStage = 3;
    } else {
      pAnim->bStage = 4;
      break;
    }
  case 3:
// Post action delay
    if ( --pAnim->uintCtr == 0 ) {
      pAnim->bStage = 4;
    }
    break;
  case 4:
// Next step in sequence
    pAnim->bIndex++;
    pAnim->bStage = 0;
    break;
  }
  serviceFaders();
}

/**
 * Interrupt handler
 */
void interrupt() {
  playAnimation(&animTest);
  
  TMR0H = 0xF9;    // preset for Timer0 MSB register
  TMR0L = 0xC0;    // preset for Timer0 LSB register
  TMR0IF_bit = 0;
}
/**
 * Entry point
 */
void main() {

  int i, intSteps;
// Create fader schedule
  intSteps = createSchedule(5, 0);
// Create the faders
  for( i=0; i<animTest.bTotalSteps; i++ ) {
    outputScheduleDef* pStep;
    pStep = &aryLEDseq[i];
    createFader(&pStep->fader, animTest.pPort, pStep->bPin,
                      (uint)intSteps, TRUE, FALSE,
                      FALSE, FALSE, TRUE, NULL);
  }
// Set-up hardware
  ANSELC = 0;                    // Configure PORTC pins as digital
  TRISC = 0;                     // Configure PORTC as output
  LATC = 0;                      // Clear PORTC
// Timer0 Registers:
// 16-Bit Mode; Prescaler=1:1;
  T0CON.TMR0ON = 1;// On/Off Control bit, 1=Enable, 0=Disable
  T0CON.T08BIT = 0;// 8-bit/16-bit Control bi: 0=16-bit timer/counter
  T0CON.T0CS   = 0;// Clock Source Select bit: 0=Internal Clock (CLKO)
  T0CON.T0SE   = 0;// Source Edge Select bit: 0=low/high
  T0CON.PSA    = 1;// Prescaler Assignment bit: 1=NOT assigned/bypassed
  T0CON.T0PS2  = 0;// bits 2-0  PS2:PS0: Prescaler Select bits
  T0CON.T0PS1  = 0;
  T0CON.T0PS0  = 0;
// TMRH Preset=F9; TMRL Preset=C0; Freq=5,000.00Hz; Period=200,000 ns
  TMR0H = 0xF9;    // preset for Timer0 MSB register
  TMR0L = 0xC0;    // preset for Timer0 LSB register
  TMR0IE_bit = 1;
// Global interrupt enable
  GIE_bit = 1;

  while( 1 ) {
// Loop forever
  }
}